/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.data.engine.api.DataEngineContext;
import org.eclipse.birt.data.engine.api.IBaseQueryResults;
import org.eclipse.birt.data.engine.api.IQueryDefinition;
import org.eclipse.birt.data.engine.api.IQueryResults;
import org.eclipse.birt.data.engine.api.IResultMetaData;
import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.executor.BaseQuery;
import org.eclipse.birt.data.engine.executor.cache.ResultSetCache;
import org.eclipse.birt.data.engine.expression.ExpressionProcessor;
import org.eclipse.birt.data.engine.impl.aggregation.AggregateTable;
import org.eclipse.birt.data.engine.impl.document.FilterDefnUtil;
import org.eclipse.birt.data.engine.impl.document.QueryResultIDManager;
import org.eclipse.birt.data.engine.impl.document.QueryResultIDUtil;
import org.eclipse.birt.data.engine.impl.document.QueryResultInfo;
import org.eclipse.birt.data.engine.impl.document.RDLoad;
import org.eclipse.birt.data.engine.impl.document.RDUtil;
import org.eclipse.birt.data.engine.impl.document.SubQueryIndexUtil;
import org.eclipse.birt.data.engine.impl.document.stream.StreamManager;
import org.eclipse.birt.data.engine.impl.document.viewing.CachedExprResultSet;
import org.eclipse.birt.data.engine.impl.document.viewing.ExprMetaUtil;
import org.eclipse.birt.data.engine.impl.document.viewing.JSIVRowObject;
import org.eclipse.birt.data.engine.impl.document.viewing.NewInstanceHelper;
import org.eclipse.birt.data.engine.impl.document.viewing.ViewingUtil;
import org.eclipse.birt.data.engine.odi.IDataSource;
import org.eclipse.birt.data.engine.odi.IEventHandler;
import org.eclipse.birt.data.engine.odi.IQuery;
import org.eclipse.birt.data.engine.odi.IResultIterator;
import org.eclipse.birt.data.engine.odi.IResultObject;
import org.eclipse.birt.data.engine.odi.IResultObjectEvent;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;

/**
 * Query for viewing
 */
class PreparedIVQuery extends PreparedDataSourceQuery
{
	private String rootQueryResultID;
	private String parentQueryResultID;
	private String selfQueryResultID;
	
	//
	private DataEngineContext dataEngineContext;
	private DataEngineSession session;
	
	private Map subQueryUtilMap;
	/**
	 * @param dataEngine
	 * @param queryDefn
	 * @throws DataException
	 */
	PreparedIVQuery( DataEngineImpl dataEngine, IQueryDefinition queryDefn )
			throws DataException
	{
		super( dataEngine, queryDefn, null, null );
		Object[] params = {
				dataEngine, queryDefn
		};
		logger.entering( PreparedIVQuery.class.getName( ),
				"PreparedIVQuery",
				params );
		
		this.dataEngineContext = dataEngine.getContext( );
		this.session = dataEngine.getSession( );
				
		String basedQueryResultID = queryDefn.getQueryResultsID( );
		rootQueryResultID = QueryResultIDUtil.get1PartID( basedQueryResultID );
		if ( rootQueryResultID == null )
			rootQueryResultID = basedQueryResultID;
		else
			parentQueryResultID = QueryResultIDUtil.get2PartID( basedQueryResultID );

		this.subQueryUtilMap = new HashMap();
		
		selfQueryResultID = QueryResultIDManager.getNextID( session,
				rootQueryResultID );
		logger.exiting( PreparedIVQuery.class.getName( ), "PreparedIVQuery" );
	}
	
	/*
	 * @see org.eclipse.birt.data.engine.api.IPreparedQuery#getParameterMetaData()
	 */
	public Collection getParameterMetaData( ) throws BirtException
	{
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.birt.data.engine.impl.PreparedDataSourceQuery#produceQueryResults(org.eclipse.birt.data.engine.api.IBaseQueryResults, org.mozilla.javascript.Scriptable)
	 */
	protected IQueryResults produceQueryResults( IBaseQueryResults outerResults, Scriptable scope ) throws DataException
	{
		QueryResults queryResults = (QueryResults) super.produceQueryResults( outerResults,
				scope );
		queryResults.setID( rootQueryResultID, selfQueryResultID );
		return queryResults;
	}
	
	/*
	 * @see org.eclipse.birt.data.engine.impl.PreparedDataSourceQuery#execSubquery(org.eclipse.birt.data.engine.odi.IResultIterator,
	 *      java.lang.String, org.mozilla.javascript.Scriptable)
	 */
	public IQueryResults execSubquery( IResultIterator iterator, IQueryExecutor parentExecutor,
			String subQueryName, Scriptable subScope ) throws DataException
	{
		this.prepareSubQueryUtil( iterator, subQueryName );

		PreparedIVSubQuery preparedSubIVQuery = new PreparedIVSubQuery( dataEngine,
				subScope,
				queryDefn,
				rootQueryResultID,
				preparedQuery.getSubQueryDefn( subQueryName ),
				preparedQuery.getSubQueryLevel( subQueryName ),
				iterator.getCurrentResultIndex( ),
				iterator.getResultClass(),
				(SubQueryIndexUtil) this.subQueryUtilMap.get(subQueryName) );
		return preparedSubIVQuery.execute( this.session.getSharedScope( ));
	}
	
	/**
	 * @param iterator
	 * @param subQueryName
	 * @throws DataException
	 */
	private void prepareSubQueryUtil( IResultIterator iterator,
			String subQueryName ) throws DataException
	{
		if ( this.subQueryUtilMap.get( subQueryName ) != null )
			return;

		int groupLevel = preparedQuery.getSubQueryLevel( subQueryName );
		int[] groupStartAndEndIndex = iterator.getGroupStartAndEndIndex( groupLevel );
		int[] validRowIndex = populateValidRowID( iterator );
		RDLoad rdLoad = RDUtil.newLoad( session.getTempDir( ), dataEngineContext,
				new QueryResultInfo( null,
						null,
						rootQueryResultID,
						null,
						-1 ) );
		SubQueryIndexUtil subQueryUtil = new SubQueryIndexUtil( rdLoad.loadRootGroupUtil( )
				.getGroupStartAndEndIndex( groupLevel ),
				groupStartAndEndIndex,
				validRowIndex );
		this.subQueryUtilMap.put( subQueryName, subQueryUtil );
	}
	
	/**
	 * 
	 * @param odiResult
	 * @throws DataException
	 */
	private int[] populateValidRowID( IResultIterator odiResult )
			throws DataException
	{
		int[] validRowIndex = new int[odiResult.getRowCount( )];
		ResultSetCache rsCache = odiResult.getResultSetCache( );
		int currIndex = rsCache.getCurrentIndex( );
		rsCache.reset( );
		for ( int i = 0; i < validRowIndex.length; i++ )
		{
			rsCache.next( );
			validRowIndex[i] = ( (Integer) rsCache.getCurrentResult( )
					.getFieldValue( ExprMetaUtil.POS_NAME ) ).intValue( );
		}
		rsCache.moveTo( currIndex );

		return validRowIndex;
	}
	
	/*
	 * @see org.eclipse.birt.data.engine.impl.PreparedQuery#newExecutor()
	 */
	protected QueryExecutor newExecutor( )
	{
		Scriptable queryScope;
		queryScope = dataEngineContext.getScriptContext( )
				.getContext( )
				.newObject( this.session.getSharedScope( ) );
		queryScope.setParentScope( this.session.getSharedScope( ) );
		queryScope.setPrototype( this.session.getSharedScope( ) );
		return new IVExecutor( queryScope );
	}
	
	/**
	 * Excutor for viewing
	 */
	private class IVExecutor extends QueryExecutor
	{
		/** */
		private Scriptable queryScope;		
		private JSIVRowObject ivRowObject;
		
		public IVExecutor( Scriptable queryScope )
		{
			super( queryScope, queryDefn, new AggregateTable( session.getTempDir( )), session );
			this.queryScope = queryScope;
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#createOdiDataSource()
		 */
		protected IDataSource createOdiDataSource( ) throws DataException
		{
			return NewInstanceHelper.newDataSource( );
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#findDataSource()
		 */
		protected DataSourceRuntime findDataSource( ) throws DataException
		{			
			return NewInstanceHelper.newDataSourceRuntime( queryScope );
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#newDataSetRuntime()
		 */
		protected DataSetRuntime newDataSetRuntime( ) throws DataException
		{
			return new DataSetRuntime( NewInstanceHelper.newDataSetDesign( ),
					this, this.getSession( ) ) {
				/*
				 * @see org.eclipse.birt.data.engine.impl.DataSetRuntime#getScriptScope()
				 */
				public Scriptable getScriptScope( )
				{
					return queryScope;
				}
			};
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#createOdiQuery()
		 */
		protected IQuery createOdiQuery( ) throws DataException
		{
			return NewInstanceHelper.newOdiQuery( );
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#populateOdiQuery()
		 */
		protected void populateOdiQuery( )
		{
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.QueryExecutor#getQueryScope()
		 */
		public Scriptable getQueryScope( )
		{
			return queryScope;
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.QueryExecutor#getResultMetaData()
		 */
		public IResultMetaData getResultMetaData( ) throws DataException
		{
			RDLoad rdLoad = RDUtil.newLoad( session.getTempDir( ), dataEngineContext,
					new QueryResultInfo( rootQueryResultID,
							parentQueryResultID,
							selfQueryResultID,
							null,
							-1 ) );
			return rdLoad.loadResultMetaData( );
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#executeOdiQuery()
		 */
		protected IResultIterator executeOdiQuery( IEventHandler eventHandler, StopSign stopSign )
				throws DataException
		{
			RDLoad rdLoad = null;
			List realList = null;
			
			if ( parentQueryResultID != null )
			{
				if ( basedOnParentRS( ) )
				{
					rdLoad = RDUtil.newLoad( session.getTempDir( ), dataEngineContext,
							new QueryResultInfo( rootQueryResultID,
									parentQueryResultID,
									selfQueryResultID,
									null,
									-1 ) );
					List parentList = rdLoad.loadFilterDefn( StreamManager.ROOT_STREAM,
							StreamManager.PARENT_SCOPE );
					realList = FilterDefnUtil.getRealFilterList( parentList,
							queryDefn.getFilters( ) );
				}
			}

			if ( rdLoad == null )
			{
				parentQueryResultID = null;
				rdLoad = RDUtil.newLoad( session.getTempDir( ), dataEngineContext,
						new QueryResultInfo( rootQueryResultID,
								null,
								selfQueryResultID,
								null,
								-1 ) );
				List rootList = rdLoad.loadFilterDefn( StreamManager.ROOT_STREAM,
						StreamManager.BASE_SCOPE );
				realList = FilterDefnUtil.getRealFilterList( rootList,
						queryDefn.getFilters( ) );
			}
			
			this.ivRowObject = new JSIVRowObject( );
			this.queryScope.put( "row", this.queryScope, ivRowObject );
			
			CachedExprResultSet resultIterator = new CachedExprResultSet( getBaseQuery( realList ),
					rdLoad.loadExprDataResultSet( ),
					rdLoad.loadRootGroupUtil( ), session, stopSign );
			resultIterator.setJSRowObject( ivRowObject );
			
			this.ivRowObject.setResultSetIterator( resultIterator );
			
			return resultIterator;
		}
		
		/**
		 * @return
		 * @throws DataException
		 */
		private boolean basedOnParentRS( ) throws DataException
		{
			RDLoad rdLoad = RDUtil.newLoad( session.getTempDir( ), dataEngineContext,
					new QueryResultInfo( rootQueryResultID,
							parentQueryResultID,
							selfQueryResultID,
							null,
							-1 ) );
			List parentList = rdLoad.loadFilterDefn( StreamManager.ROOT_STREAM,
					StreamManager.PARENT_SCOPE );
			List selfList = queryDefn.getFilters( );
			return !FilterDefnUtil.isConflictFilter( parentList, selfList );
		}
		
		/**
		 * @return
		 * @throws DataException
		 */
		private BaseQuery getBaseQuery( List filterList )
				throws DataException
		{
			BaseQuery baseQuery = new BaseQuery( ) {				
				/*
				 * @see org.eclipse.birt.data.engine.odi.IQuery#close()
				 */
				public void close( )
				{
				}
			};

			baseQuery.addOnFetchEvent( new ResultObjectEvent( this.ivRowObject ) );
			
			IQueryDefinition queryDefn = PreparedIVQuery.this.queryDefn;

			//---support group
			// since group is not supported in 2.1, it is not considered for now.
			
			//---support sort
			baseQuery.setOrdering( ViewingUtil.getSortList( queryDefn.getSorts( ),
					session.getEngineContext( )
							.getScriptContext( )
							.getContext( ) ) );
			
			//---support filter
			if ( filterList.size( ) > 0 )
			{
				IResultObjectEvent objectEvent = new FilterByRow( new ArrayList( ),
						filterList,
						new ArrayList( ),
						new ArrayList( ),
						new ArrayList( ),
						dataSet );
				baseQuery.addOnFetchEvent( objectEvent );
			}
			
			baseQuery.setExprProcessor( new ExpressionProcessor( this.dataSet ) );
			
			return baseQuery;
		}
	}
	
	/**
	 * This event can be used to get current result object and then set to its
	 * bound java script object
	 */
	private static class ResultObjectEvent implements IResultObjectEvent
	{
		//
		private JSIVRowObject rowObject;

		/**
		 * @param rowObject
		 */
		private ResultObjectEvent( JSIVRowObject rowObject )
		{
			this.rowObject = rowObject;
		}

		/*
		 * @see org.eclipse.birt.data.engine.odi.IResultObjectEvent#process(org.eclipse.birt.data.engine.odi.IResultObject,
		 *      int)
		 */
		public boolean process( IResultObject resultObject, int rowIndex )
				throws DataException
		{
			this.rowObject.setResultObject( resultObject );
			return true;
		}
	}
	
}
