/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.data.engine.impl;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.core.util.IOUtil;
import org.eclipse.birt.data.engine.api.DataEngineContext;
import org.eclipse.birt.data.engine.api.IQueryDefinition;
import org.eclipse.birt.data.engine.api.ISubqueryDefinition;
import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.executor.cache.ResultSetCache;
import org.eclipse.birt.data.engine.i18n.ResourceConstants;
import org.eclipse.birt.data.engine.impl.aggregation.AggregateTable;
import org.eclipse.birt.data.engine.impl.document.QueryResultInfo;
import org.eclipse.birt.data.engine.impl.document.RDGroupUtil;
import org.eclipse.birt.data.engine.impl.document.RDLoadUtil;
import org.eclipse.birt.data.engine.impl.document.StreamWrapper;
import org.eclipse.birt.data.engine.impl.document.SubQueryIndexUtil;
import org.eclipse.birt.data.engine.impl.document.stream.StreamManager;
import org.eclipse.birt.data.engine.impl.document.viewing.CachedSubExprResultSet;
import org.eclipse.birt.data.engine.impl.document.viewing.JSIVRowObject;
import org.eclipse.birt.data.engine.impl.document.viewing.NewInstanceHelper;
import org.eclipse.birt.data.engine.odi.IDataSource;
import org.eclipse.birt.data.engine.odi.IEventHandler;
import org.eclipse.birt.data.engine.odi.IQuery;
import org.eclipse.birt.data.engine.odi.IResultClass;
import org.eclipse.birt.data.engine.odi.IResultIterator;
import org.eclipse.birt.data.engine.odi.IResultObject;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;

/**
 * 
 */
public class PreparedIVSubQuery extends PreparedDataSourceQuery
		implements
			IPreparedQueryService
{
	private String subQueryName;
	
	private String basedQueryResultID;
	
	//
	private Scriptable sharedScope;	
	private DataEngineContext dataEngineContext;
	private DataEngineSession session;
	
	private int parentIndex;
	
	private SubQueryIndexUtil subQueryManager;
	
	private int groupLevel;
	private IResultClass resultClass;
	/**
	 * 
	 * @param dataEngine
	 * @param scope
	 * @param queryDefn
	 * @param basedQueryResultID
	 * @param subQueryDefn
	 * @param groupLevel
	 * @param parentIndex
	 * @param subQueryManager
	 * @throws DataException
	 */
	PreparedIVSubQuery( DataEngineImpl dataEngine, Scriptable scope,
			IQueryDefinition queryDefn, String basedQueryResultID,
			ISubqueryDefinition subQueryDefn, int groupLevel, int parentIndex, IResultClass resultClass,
			SubQueryIndexUtil subQueryManager ) throws DataException
	{
		super( dataEngine, subQueryDefn, queryDefn, null, null );
		Object[] params = {
				dataEngine,
				scope,
				queryDefn,
				basedQueryResultID,
				subQueryDefn,
				new Integer( groupLevel ),
				new Integer( parentIndex ),
				subQueryManager
		};
		logger.entering( PreparedIVSubQuery.class.getName( ),
				"PreparedIVSubQuery",
				params );

		this.dataEngineContext = dataEngine.getContext( );
		this.session = dataEngine.getSession( );
		this.basedQueryResultID = basedQueryResultID;
		this.subQueryName = subQueryDefn.getName( );
		this.parentIndex = parentIndex;

		this.sharedScope = scope;
		this.subQueryManager = subQueryManager;
		this.resultClass = resultClass;
		this.groupLevel = groupLevel;
		logger.exiting( PreparedIVSubQuery.class.getName( ),
				"PreparedIVSubQuery" );
	}

	/**
	 * @return
	 */
	public int getGroupLevel( )
	{
		return this.groupLevel;
	}
	
	/*
	 * @see org.eclipse.birt.data.engine.api.IPreparedQuery#getParameterMetaData()
	 */
	public Collection getParameterMetaData( ) throws BirtException
	{
		return null;
	}
	
	/*
	 * @see org.eclipse.birt.data.engine.impl.PreparedDataSourceQuery#newExecutor()
	 */
	protected QueryExecutor newExecutor( )
	{
		Scriptable queryScope;
		queryScope = dataEngineContext.getScriptContext( ).getContext( ).newObject( sharedScope );
		queryScope.setParentScope( sharedScope );
		queryScope.setPrototype( sharedScope );
		
		return new IVExecutor( queryScope );
	}
	
	/**
	 * Excutor for viewing
	 */
	private class IVExecutor extends QueryExecutor
	{
		/** */
		private Scriptable queryScope;		
		private JSIVRowObject ivRowObject;
		
		public IVExecutor( Scriptable queryScope )
		{
			super( queryScope, queryDefn, new AggregateTable( session.getTempDir( ) ), session );
			this.queryScope = queryScope;
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#createOdiDataSource()
		 */
		protected IDataSource createOdiDataSource( ) throws DataException
		{
			return NewInstanceHelper.newDataSource( );
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#findDataSource()
		 */
		protected DataSourceRuntime findDataSource( ) throws DataException
		{			
			return NewInstanceHelper.newDataSourceRuntime( queryScope );
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#newDataSetRuntime()
		 */
		protected DataSetRuntime newDataSetRuntime( ) throws DataException
		{
			return new DataSetRuntime( NewInstanceHelper.newDataSetDesign( ),
					this, this.getSession( ) ) {
				/*
				 * @see org.eclipse.birt.data.engine.impl.DataSetRuntime#getScriptScope()
				 */
				public Scriptable getScriptScope( )
				{
					return queryScope;
				}
			};
		}

		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#createOdiQuery()
		 */
		protected IQuery createOdiQuery( ) throws DataException
		{
			return NewInstanceHelper.newOdiQuery( );
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#populateOdiQuery()
		 */
		protected void populateOdiQuery( )
		{
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.QueryExecutor#getQueryScope()
		 */
		public Scriptable getQueryScope( )
		{
			return queryScope;
		}
		
		/*
		 * @see org.eclipse.birt.data.engine.impl.PreparedQuery.Executor#executeOdiQuery()
		 */
		protected IResultIterator executeOdiQuery( IEventHandler eventHandler, StopSign stopSign )
				throws DataException
		{			
			if( parentIndex == -1 )
				return new IResultIterator(){

					public void close( )
					{
						// TODO Auto-generated method stub
						
					}

					/*
					 * (non-Javadoc)
					 * @see org.eclipse.birt.data.engine.odi.IResultIterator#doSave(org.eclipse.birt.data.engine.impl.document.StreamWrapper, boolean)
					 */
					public void doSave( StreamWrapper streamsWrapper,
							boolean isSubQuery ) throws DataException
					{
						//When subquery result is empty, we should still write dummy streams
						writeEmptyStream( streamsWrapper.getStreamForResultClass( ) );
						writeEmptyStream( streamsWrapper.getStreamForDataSet( ) );
						writeEmptyStream( streamsWrapper.getStreamForDataSetRowLens( ) );
						writeEmptyStream( streamsWrapper.getStreamForParentIndex( ) );
						if ( streamsWrapper.getStreamForRowIndexInfo( ) != null )
						{
							try 
							{
								streamsWrapper.getStreamForRowIndexInfo().close();
							}
							catch (IOException e) 
							{
								throw new DataException( e.getMessage() );
							}
						}
						writeEmptyStream( streamsWrapper.getStreamForGroupInfo( ) );

					}

					/**
					 * 
					 * @param stream
					 * @throws DataException
					 */
					private void writeEmptyStream( OutputStream stream )
							throws DataException
					{
						try
						{
							if ( stream != null )
							{
								IOUtil.writeInt( stream, 0 );
								stream.close( );
							}
						}
						catch ( IOException e )
						{
							throw new DataException( ResourceConstants.RD_SAVE_ERROR,
									e );
						}

					}

					public void first( int groupingLevel ) throws DataException
					{
						// TODO Auto-generated method stub
						
					}

					public int getCurrentGroupIndex( int groupLevel ) throws DataException
					{
						// TODO Auto-generated method stub
						return 0;
					}

					public IResultObject getCurrentResult( ) throws DataException
					{
						// TODO Auto-generated method stub
						return null;
					}

					public int getCurrentResultIndex( ) throws DataException
					{
						// TODO Auto-generated method stub
						return -1;
					}

					public int getEndingGroupLevel( ) throws DataException
					{
						// TODO Auto-generated method stub
						return 0;
					}

					public IExecutorHelper getExecutorHelper( )
					{
						// TODO Auto-generated method stub
						return null;
					}

					public int[] getGroupStartAndEndIndex( int groupLevel ) throws DataException
					{
						// TODO Auto-generated method stub
						return null;
					}

					public IResultClass getResultClass( ) throws DataException
					{
						return resultClass;
					}

					public ResultSetCache getResultSetCache( )
					{
						// TODO Auto-generated method stub
						return null;
					}

					public int getRowCount( ) throws DataException
					{
						// TODO Auto-generated method stub
						return 0;
					}

					public int getStartingGroupLevel( ) throws DataException
					{
						// TODO Auto-generated method stub
						return 0;
					}

					public void last( int groupingLevel ) throws DataException
					{
						// TODO Auto-generated method stub
						
					}

					public boolean next( ) throws DataException
					{
						// TODO Auto-generated method stub
						return false;
					}

					public Object getAggrValue( String aggrName )
							throws DataException
					{
						// TODO Auto-generated method stub
						return null;
					}};
			StreamManager streamManager = new StreamManager( dataEngineContext,
					new QueryResultInfo( basedQueryResultID,
							subQueryName,
							subQueryManager.getOldGroupInstanceIndex( parentIndex ) ) );
			
			RDGroupUtil rdGroupUtil = RDLoadUtil.loadGroupUtil( session.getTempDir( ), streamManager,
					StreamManager.SUB_QUERY_STREAM,
					StreamManager.ROOT_STREAM );	
			CachedSubExprResultSet resultIterator = new CachedSubExprResultSet( streamManager,
					subQueryManager,
					parentIndex,
					rdGroupUtil, session, stopSign );
			
			ivRowObject = new JSIVRowObject( );
			this.queryScope.put( "row", this.queryScope, ivRowObject );
			ivRowObject.setResultSetIterator( resultIterator );
			
			return resultIterator;
		}
	}
	
}
