/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.impl.document.viewing;

import java.io.IOException;
import java.util.List;

import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.executor.BaseQuery;
import org.eclipse.birt.data.engine.executor.cache.CacheRequest;
import org.eclipse.birt.data.engine.executor.cache.OdiAdapter;
import org.eclipse.birt.data.engine.executor.cache.ResultSetCache;
import org.eclipse.birt.data.engine.executor.cache.SmartCache;
import org.eclipse.birt.data.engine.executor.cache.SortSpec;
import org.eclipse.birt.data.engine.i18n.ResourceConstants;
import org.eclipse.birt.data.engine.impl.DataEngineSession;
import org.eclipse.birt.data.engine.impl.ExecutorHelper;
import org.eclipse.birt.data.engine.impl.IExecutorHelper;
import org.eclipse.birt.data.engine.impl.StopSign;
import org.eclipse.birt.data.engine.impl.document.GroupInfoUtil;
import org.eclipse.birt.data.engine.impl.document.RDGroupUtil;
import org.eclipse.birt.data.engine.impl.document.StreamWrapper;
import org.eclipse.birt.data.engine.impl.document.util.IExprDataResultSet;
import org.eclipse.birt.data.engine.odi.IResultObject;
import org.mozilla.javascript.Scriptable;

/**
 * This is the IResultIterator implemenation of ODI layer for query running on
 * report document.
 */
public class CachedExprResultSet extends BaseCachedResultSet
{	
	private DataEngineSession session;
	
	/**
	 * @param query
	 * @param meta
	 * @param exprDataResultSet
	 * @param stopSign
	 * @throws DataException
	 */
	public CachedExprResultSet( BaseQuery query,
			IExprDataResultSet exprDataResultSet, RDGroupUtil rdGroupUtil,
			DataEngineSession session, StopSign stopSign ) throws DataException
	{
		this.resultClass = exprDataResultSet.getResultClass( );				
		this.rdGroupUtil = rdGroupUtil;
		this.session = session;
		this.doPrepare( query, exprDataResultSet, stopSign );
		this.next( );
	}
	
	/**
	 * @param query
	 * @param exprDataResultSet
	 * @param stopSign
	 * @throws DataException
	 */
	private void doPrepare( BaseQuery query,
			IExprDataResultSet exprDataResultSet, StopSign stopSign ) throws DataException
	{
		// 1: filtering		
		smartCache = new SmartCache( new CacheRequest( query.getMaxRows( ),
				query.getFetchEvents( ),
				null,
				new DummyEventHandler( ) ),
				new OdiAdapter( exprDataResultSet ),
				resultClass, this.session, stopSign );
		
		// here rdGroupUtil must be replaced with updated rdGroupUtil.
		this.updateGroupUtil( smartCache );
		
		this.rowCount = smartCache.getCount( );
		this.rdGroupUtil.setCacheProvider( this );

		// 2 sorting on the row of group
		SortSpec sortSpec = ViewingUtil.getSortSpec( null,
				query.getOrdering( ),
				resultClass );
		if ( sortSpec != null && sortSpec.length( ) > 0 )
		{
			int innerGroupLevel = rdGroupUtil.getGroupLevel( );
			int[] groupRowIndex = rdGroupUtil.getGroupStartAndEndIndex( innerGroupLevel );
			
			smartCache = new SmartCache( new CacheRequest( 0, null, null, new DummyEventHandler() ),
					new OdiAdapter( new ExprTotalDataResultSet( smartCache,
							this.resultClass,
							sortSpec,
							groupRowIndex,
							this.rowCount, this.session ) ),
					this.resultClass, this.session, stopSign );
		}
		
		exprDataResultSet.close( );
	}
	
	/**
	 * @param smartCache
	 * @throws DataException
	 */
	private void updateGroupUtil( ResultSetCache smartCache ) throws DataException
	{		
		int[] validIndex = new int[smartCache.getCount( )];
		int indexColumn = resultClass.getFieldCount( );
		int index = 0;
		while ( smartCache.next( ) )
		{
			IResultObject resultObject = smartCache.getCurrentResult( );
			validIndex[index++] = ( (Integer) resultObject.getFieldValue( indexColumn ) ).intValue( );
		}
		smartCache.reset( );

		List[] oldGroups = rdGroupUtil.getGroups( );
		List[] newGroups = GroupInfoUtil.getGroupInfo( oldGroups, validIndex );
		rdGroupUtil.setGroups( newGroups );
	}
	
	//-------------------------IResultIterator---------------------------------
		
	/*
	 * @see org.eclipse.birt.data.engine.odi.IResultIterator#doSave(org.eclipse.birt.data.engine.impl.document.StreamWrapper,
	 *      boolean)
	 */
	public void doSave( StreamWrapper streamsWrapper, boolean isSubQuery )
			throws DataException
	{
		// save the row index information
		int currIndex = this.getCurrentIndex( );
		this.smartCache.moveTo( 0 );
		this.saveExprResult( streamsWrapper );
		this.smartCache.moveTo( currIndex );
		
		// save the group information
		try
		{
			this.rdGroupUtil.saveGroupsToStream( streamsWrapper.getStreamForGroupInfo( ) );
		}
		catch ( IOException e )
		{
			throw new DataException( ResourceConstants.RD_SAVE_ERROR,
					e,
					"Group Information" );
		}
	}
	
	/**
	 * @param streamsWrapper
	 * @throws DataException
	 */
	private void saveExprResult( StreamWrapper streamsWrapper )
			throws DataException
	{
		int idColumnPos = ExprMetaUtil.getIdColumnPos( resultClass );
		
		RowIndexUtil rowInfoUtil = new RowIndexUtil( streamsWrapper.getStreamForRowIndexInfo( ) );
		for ( int i = 0; i < this.getCount( ); i++ )
		{
			IResultObject resultObject = this.smartCache.getCurrentResult( );

			int index = ( (Integer) resultObject.getFieldValue( idColumnPos ) ).intValue( );
			rowInfoUtil.write( index );

			this.smartCache.next( );
		}
		
		// part2
		rowInfoUtil.close( );
	}
	
	private Scriptable jsRowObject;
	private ExecutorHelper executorHelper;

	/**
	 * @param
	 */
	public void setJSRowObject( Scriptable jsRowObject )
	{
		this.jsRowObject = jsRowObject;
	}

	/*
	 * @see org.eclipse.birt.data.engine.impl.document.viewing.BaseCachedResultSet#getExecutorHelper()
	 */
	public IExecutorHelper getExecutorHelper( )
	{
		if ( executorHelper == null )
		{
			executorHelper = new ExecutorHelper( null );
			executorHelper.setScriptable( jsRowObject );
		}
		return executorHelper;
	}
	
}
