/*
 *************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.birt.data.engine.olap.data.impl.aggregation;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.i18n.DataResourceHandle;
import org.eclipse.birt.data.engine.i18n.ResourceConstants;
import org.eclipse.birt.data.engine.olap.data.api.DimLevel;
import org.eclipse.birt.data.engine.olap.data.api.IAggregationResultSet;
import org.eclipse.birt.data.engine.olap.data.api.MeasureInfo;
import org.eclipse.birt.data.engine.olap.data.impl.DimColumn;
import org.eclipse.birt.data.engine.olap.data.impl.dimension.Member;

/**
 * The data prepared for aggregation is from a <code>IAggregationResultSet</code> instance
 *
 */
public class DataSetFromAggregationResultSet implements IDataSet4Aggregation
{

	private IAggregationResultSet aggregationResultSet;
	
	//Dummy dimensions from dimLevles in  aggregationResultSet
	//the dimIndex, levelIndex are got from it
	DummyDimension[] dimensions;

	private int currentCursor = -1;

	public DataSetFromAggregationResultSet(
			IAggregationResultSet aggregationResultSet )
	{
		this.aggregationResultSet = aggregationResultSet;
		initDummyDimensions( );
	}

	private void initDummyDimensions( )
	{
		DimLevel[] dimLevels = aggregationResultSet.getAllLevels( );
		if ( dimLevels == null || dimLevels.length == 0 )
		{
			dimensions = new DummyDimension[0];
			return;
		}
		Map<String, Set<String>> dimNameToLevelNamesMap = new HashMap<String, Set<String>>( );
		for ( DimLevel dl : dimLevels )
		{
			String dimName = dl.getDimensionName( );
			Set<String> levelNames = dimNameToLevelNamesMap.get( dimName );
			if ( levelNames == null )
			{
				levelNames = new HashSet<String>( );
				dimNameToLevelNamesMap.put( dimName, levelNames );
			}
			levelNames.add( dl.getLevelName( ) );
		}
		dimensions = new DummyDimension[dimNameToLevelNamesMap.keySet( ).size( )];
		int i = 0;
		for ( String dimName : dimNameToLevelNamesMap.keySet( ) )
		{
			Set<String> levelNameSet = dimNameToLevelNamesMap.get( dimName );
			String[] levelNames = levelNameSet.toArray( new String[0] );
			DummyDimension dd = new DummyDimension( dimName, levelNames );
		
			int[] indexesInAggregationResultSet = new int[levelNames.length];
			for (int j = 0; j < indexesInAggregationResultSet.length; j++  )
			{
				DimLevel dl = new DimLevel(dimName, levelNames[j]);
				indexesInAggregationResultSet[j] = aggregationResultSet.getLevelIndex( dl );
			}
			dd.setDimLevelIndexesInAggregationResultSet( indexesInAggregationResultSet );
			dimensions[i++] = dd;
		}
	}

	private DummyDimension getDimension( String dimensionName )
	{
		for ( int i = 0; i < dimensions.length; i++ )
		{
			if ( dimensions[i].getDimensionName( ).equals( dimensionName ) )
			{
				return dimensions[i];
			}
		}
		return null;
	}

	private DummyDimension getDimension( int dimIndex )
	{
		return dimensions[dimIndex];
	}

	private int getDimensionIndex( String dimensionName )
	{
		for ( int i = 0; i < dimensions.length; i++ )
		{
			if ( dimensions[i].getDimensionName( ).equals( dimensionName ) )
			{
				return i;
			}
		}
		return -1;
	}

	private int getInnerIndexInAggregationResultSet( int dimIndex,
			int levelIndex )
	{
		DummyDimension dd = DataSetFromAggregationResultSet.this.getDimension( dimIndex );
		return dd.getIndexInAggregationResultSet( levelIndex );
	}

	public Object getMeasureValue( int measureIndex ) throws IOException
	{
		return aggregationResultSet.getAggregationValue( measureIndex );
	}

	public Member getMember( int dimIndex, int levelIndex )
			throws BirtException, IOException
	{
		int index = getInnerIndexInAggregationResultSet( dimIndex, levelIndex );
		return aggregationResultSet.getCurrentRow( ).getLevelMembers( )[index];
	}

	public MetaInfo getMetaInfo( )
	{
		return new IDataSet4Aggregation.MetaInfo( ) {

			public String[] getAttributeNames( int dimIndex, int levelIndex )
			{
				int index = getInnerIndexInAggregationResultSet( dimIndex,
						levelIndex );
				return aggregationResultSet.getAggributeNames( )[index];
			}

			public ColumnInfo getColumnInfo( DimColumn dimColumn )
					throws DataException
			{
				String dimensionName = dimColumn.getDimensionName( );
				String levelName = dimColumn.getLevelName( );
				String columnName = dimColumn.getColumnName( );

				int dimIndex = DataSetFromAggregationResultSet.this.getDimensionIndex( dimensionName );
				if ( dimIndex < 0 )
				{
					throw new DataException( DataResourceHandle.getInstance( )
							.getMessage( ResourceConstants.NONEXISTENT_DIMENSION )
							+ dimensionName );
				}

				DummyDimension dd = DataSetFromAggregationResultSet.this.getDimension( dimensionName );
				int levelIndex = dd.getLevelIndex( levelName );
				if ( levelIndex < 0 )
				{
					throw new DataException( DataResourceHandle.getInstance( )
							.getMessage( ResourceConstants.NONEXISTENT_LEVEL )
							+ "<" + dimensionName + " , " + levelName + ">" );
				}

				DimLevel dl = new DimLevel( dimensionName, levelName );
				int index = aggregationResultSet.getLevelIndex( dl );

				int columnIndex = -1;
				boolean isKey = false;
				String[] keys = aggregationResultSet.getKeyNames( )[index];
				for ( int i = 0; i < keys.length; i++ )
				{
					if ( keys[i].equals( columnName ) )
					{
						columnIndex = i;
						isKey = true;
						break;
					}
				}
				if ( !isKey )
				{
					String[] attrs = aggregationResultSet.getAggributeNames( )[index];
					for ( int i = 0; i < attrs.length; i++ )
					{
						if ( attrs[i].equals( columnName ) )
						{
							columnIndex = i;
							break;
						}
					}
				}
				if ( columnIndex < 0 )
				{
					throw new DataException( DataResourceHandle.getInstance( )
							.getMessage( ResourceConstants.NONEXISTENT_KEY_OR_ATTR )
							+ "<"
							+ dimensionName
							+ " , "
							+ levelName
							+ " , "
							+ columnName + ">" );
				}
				return new IDataSet4Aggregation.ColumnInfo( dimIndex,
						levelIndex,
						columnIndex,
						isKey );
			}

			public int getDimensionIndex( String dimensionName )
			{
				return DataSetFromAggregationResultSet.this.getDimensionIndex( dimensionName );
			}

			public String[] getKeyNames( int dimIndex, int levelIndex )
			{
				int index = getInnerIndexInAggregationResultSet( dimIndex,
						levelIndex );
				return aggregationResultSet.getKeyNames( )[index];
			}

			public int getLevelIndex( String dimensionName, String levelName )
			{
				DummyDimension dd = DataSetFromAggregationResultSet.this.getDimension( dimensionName );
				if ( dd != null )
				{
					return dd.getLevelIndex( levelName );
				}
				return -1;
			}

			public int getMeasureIndex( String measureName ) throws IOException
			{
				return aggregationResultSet.getAggregationIndex( measureName );
			}

			public MeasureInfo[] getMeasureInfos( ) throws IOException
			{
				int aggrCount = aggregationResultSet.getAggregationCount( );
				MeasureInfo[] mis = new MeasureInfo[aggrCount];
				for ( int i = 0; i < aggrCount; i++ )
				{
					MeasureInfo mi = new MeasureInfo( aggregationResultSet.getAggregationName( i ),
							aggregationResultSet.getAggregationDataType( i ) );
					mis[i] = mi;
				}
				return mis;
			}

		};
	}

	public boolean next( ) throws DataException, IOException
	{
		if ( aggregationResultSet.length( ) <= 0 )
		{
			return false;
		}
		currentCursor++;
		if ( currentCursor < aggregationResultSet.length( ) )
		{
			aggregationResultSet.seek( currentCursor );
			return true;
		}
		return false;
	}

	static class DummyDimension
	{
		String dimensionName;
		
		//the level names in this dimension
		String[] levelNames;
		
		//the corresponding DimLevel index in aggregationResultSet
		//one levelName in levelNames has a corresponding dimLevelIndex from <code>AggregationResultSet.getLevelIndex( DimLevel dl )</code>
		int[] dimLevelIndexesInAggregationResultSet;
		

		public DummyDimension( String dimensionName, String[] levelNames )
		{
			this.dimensionName = dimensionName;
			this.levelNames = levelNames;
		}
		
		public void setDimLevelIndexesInAggregationResultSet(int[] indexes)
		{
			dimLevelIndexesInAggregationResultSet = indexes;
		}
		
		public int getIndexInAggregationResultSet(int levelIndex)
		{
			return dimLevelIndexesInAggregationResultSet[levelIndex];
		}

		public String getLevelName( int index )
		{
			return levelNames[index];
		}

		public int getLevelIndex( String name )
		{
			for ( int i = 0; i < levelNames.length; i++ )
			{
				if ( levelNames[i].equals( name ) )
				{
					return i;
				}
			}
			return -1;
		}

		public String getDimensionName( )
		{
			return dimensionName;
		}
	}
}
