/*******************************************************************************
 * Copyright (c)2007 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.emitter.wpml;

import java.awt.Image;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Stack;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.birt.report.engine.api.IHTMLActionHandler;
import org.eclipse.birt.report.engine.api.IReportRunnable;
import org.eclipse.birt.report.engine.api.InstanceID;
import org.eclipse.birt.report.engine.api.RenderOption;
import org.eclipse.birt.report.engine.api.script.IReportContext;
import org.eclipse.birt.report.engine.content.IAutoTextContent;
import org.eclipse.birt.report.engine.content.IBandContent;
import org.eclipse.birt.report.engine.content.ICellContent;
import org.eclipse.birt.report.engine.content.IColumn;
import org.eclipse.birt.report.engine.content.IContainerContent;
import org.eclipse.birt.report.engine.content.IContent;
import org.eclipse.birt.report.engine.content.IDataContent;
import org.eclipse.birt.report.engine.content.IForeignContent;
import org.eclipse.birt.report.engine.content.IGroupContent;
import org.eclipse.birt.report.engine.content.IHyperlinkAction;
import org.eclipse.birt.report.engine.content.IImageContent;
import org.eclipse.birt.report.engine.content.ILabelContent;
import org.eclipse.birt.report.engine.content.IListBandContent;
import org.eclipse.birt.report.engine.content.IListContent;
import org.eclipse.birt.report.engine.content.IListGroupContent;
import org.eclipse.birt.report.engine.content.IPageContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.content.IRowContent;
import org.eclipse.birt.report.engine.content.IStyle;
import org.eclipse.birt.report.engine.content.ITableBandContent;
import org.eclipse.birt.report.engine.content.ITableContent;
import org.eclipse.birt.report.engine.content.ITableGroupContent;
import org.eclipse.birt.report.engine.content.ITextContent;
import org.eclipse.birt.report.engine.content.impl.TextContent;
import org.eclipse.birt.report.engine.css.dom.ComputedStyle;
import org.eclipse.birt.report.engine.css.engine.value.birt.BIRTConstants;
import org.eclipse.birt.report.engine.css.engine.value.css.CSSConstants;
import org.eclipse.birt.report.engine.emitter.EmitterUtil;
import org.eclipse.birt.report.engine.emitter.IEmitterServices;
import org.eclipse.birt.report.engine.emitter.wpml.WpmlWriter.TextFlag;
import org.eclipse.birt.report.engine.i18n.EngineResourceHandle;
import org.eclipse.birt.report.engine.i18n.MessageConstants;
import org.eclipse.birt.report.engine.ir.EngineIRConstants;
import org.eclipse.birt.report.engine.ir.SimpleMasterPageDesign;
import org.eclipse.birt.report.engine.layout.pdf.font.FontInfo;
import org.eclipse.birt.report.engine.layout.pdf.font.FontMappingManager;
import org.eclipse.birt.report.engine.layout.pdf.font.FontMappingManagerFactory;
import org.eclipse.birt.report.engine.layout.pdf.font.FontSplitter;
import org.eclipse.birt.report.engine.layout.pdf.text.Chunk;
import org.eclipse.birt.report.engine.layout.pdf.util.HTML2Content;
import org.eclipse.birt.report.engine.presentation.ContentEmitterVisitor;
import org.eclipse.birt.report.engine.util.FlashFile;
import org.eclipse.birt.report.engine.util.SvgFile;
import org.w3c.dom.css.CSSValue;

import com.ibm.icu.util.ULocale;
import com.lowagie.text.pdf.BaseFont;

public class WordEmitterImpl
{

	protected static Logger logger = Logger.getLogger( WordEmitterImpl.class
			.getName( ) );

	protected OutputStream out = null;

	IEmitterServices eservice = null;

	protected ContentEmitterVisitor contentVisitor;

	WpmlWriter writer = null;

	public static final String OUTPUT_FORMAT = "doc";

	private int pageWidth = 0;

	private int pageHeight = 0;

	private int contentHeight = 0;

	private int contentWidth = 0;

	private int headerHeight = 0;

	private int footerHeight = 0;

	private int topMargin = 0;

	private int bottomMargin = 0;

	private int leftMargin = 0;

	private int rightMargin = 0;

	private String orientation = "portrait";

	private EmitterContext context = null;

	private IPageContent previousPage = null;

	private Stack styles = new Stack( );

	private Vector cacheCells = new Vector( );

	final static int NORMAL = -1;

	private static int firstPage = 0;

	private static int normalPage = 1;

	private int pageStatus;

	private HashSet marks = new HashSet( );

	boolean rowFilledFlag = false;

	private ArrayList<InstanceID> groupIdList = new ArrayList<InstanceID>( );

	private IReportContent reportContent;

	private static IGroupContent groupContent = null;

	private static boolean inForeign = false;

	private static boolean hasPInside = false;

	private int tocLevel = 1;
	
	private List<TocInfo> tableTocs = new ArrayList<TocInfo>( );

	public WordEmitterImpl( ContentEmitterVisitor contentVisitor )
	{
		this.contentVisitor = contentVisitor;
	}

	public void initialize( IEmitterServices service )
	{
		eservice = service;

		if ( service != null )
		{
			this.out = EmitterUtil.getOuputStream( service, "report.doc" );
		}

		writer = new WpmlWriter( out );
		context = new EmitterContext( );

		// Assume the out is always setup successfully
	}

	public String getOutputFormat( )
	{
		return WordEmitterImpl.OUTPUT_FORMAT;
	}

	public void startPage( IPageContent page )
	{
		if ( this.previousPage != null )
		{
			completePrePage( );
			previousPage = page;
			pageStatus = WordEmitterImpl.normalPage;
			context.resetWidth( );
		}
		else
		{
			previousPage = page;
			pageStatus = WordEmitterImpl.firstPage;
			if ( reportContent != null )
			{
				IContent rootContent = reportContent.getRoot( );
				if ( rootContent != null && rootContent.isRTL( ) )
					writer.start( reportContent, true );
				else
					writer.start( reportContent, false );
			}
			else
				writer.start( reportContent, false );
			handlePageBackground( );
		}
		accountPageProp( page );
		context.addWidth( contentWidth );
		writer.startPg( );
	}

	public void completePrePage( )
	{
		adjustInline( );
		writer.startPageBreak( );
		writeSection( );
		writer.endPageBreak( );
		writer.endPg( );
	}

	/**
	 * Transfer background for current page to Doc format. Now, the exported
	 * file will apply the first background properties, and followed background
	 * will ignore.
	 * 
	 * In addition, Since the Word only support fill-in background, the
	 * background attach, pos, posX, posY and repeat are not mapped to Word
	 * easyly. At present, ignore those properties.
	 * 
	 * @TODO support background properties. attach, pos, posx, posy and repeat.
	 */
	private void handlePageBackground( )
	{
		// Set the first page background which is not null to DOC
		IStyle style = previousPage.getComputedStyle( );
		String bgcolor = style.getBackgroundColor( );
		String bgimg = style.getBackgroundImage( );

		// Image priority is higher than color.
		if ( bgimg != null && !bgimg.equalsIgnoreCase( "none" ) )
		{
			try
			{
				File file = new File( bgimg );
				byte[] img = getImage( new FileInputStream( file ) );
				Base64 base = new Base64( );
				byte[] pic2Text = base.encode( img );
				writer.writePageBackground( "image", new String( pic2Text ) );
			}
			catch ( Exception e )
			{
				logger.log( Level.WARNING, "Can not get background image" );
			}
		}
		else if ( bgcolor != null && !"transparent".equalsIgnoreCase( bgcolor ) )
		{
			writer.writePageBackground( "color", bgcolor );
		}
	}

	public void end( IReportContent report )
	{
		adjustInline( );
		writeSection( );
		writer.endPg( );
		writer.end( );
	}

	private void increaseTOCLevel( IContent content )
	{
		if ( content != null && content.getTOC( ) != null )
		{
			tocLevel += 1;
		}
	}

	private void decreaseTOCLevel( IContent content )
	{
		if ( content != null && content.getTOC( ) != null )
		{
			tocLevel -= 1;
		}
	}

	public void writeSection(  )
	{
		writer.startSection( );

		if ( pageStatus == WordEmitterImpl.firstPage )
		{
			writeHeaderFooter( );
		}

		writer.writePgProp( pageHeight, pageWidth, headerHeight, footerHeight,
				topMargin, bottomMargin, leftMargin, rightMargin,
				this.orientation );

		writer.endSection( );
	}

	public void writeHeaderFooter( )
	{

		SimpleMasterPageDesign master = (SimpleMasterPageDesign) ( previousPage
				.getReportContent( ).getDesign( ).getPageSetup( )
				.getMasterPage( 0 ) );
		if ( !master.isShowHeaderOnFirst( ) )
		{
			writer.startHeader( true );
		}
		else
		{
			writer.startHeader( false );
		}

		if ( previousPage.getPageHeader( ) != null )
		{
			contentVisitor.visitChildren( previousPage.getPageHeader( ), null );
		}
		writer.endHeader( );
		writer.startFooter( );

		if ( previousPage.getPageFooter( ) != null )
		{
			contentVisitor.visitChildren( previousPage.getPageFooter( ), null );
		}
		writer.endFooter( );

	}

	public void accountPageProp( IPageContent page )
	{

		pageWidth = WordUtil.convertTo( page.getPageWidth( ), 0 );
		// 11 inch * 1440
		pageHeight = WordUtil.convertTo( page.getPageHeight( ), 0 );

		footerHeight = WordUtil.convertTo( page.getFooterHeight( ), 0 );
		headerHeight = WordUtil.convertTo( page.getHeaderHeight( ), 0 );

		topMargin = WordUtil.convertTo( page.getMarginTop( ), 0 );
		bottomMargin = WordUtil.convertTo( page.getMarginBottom( ), 0 );

		contentHeight = pageHeight - headerHeight - footerHeight - topMargin
				- bottomMargin;

		leftMargin = WordUtil.convertTo( page.getMarginLeft( ), 0 );
		rightMargin = WordUtil.convertTo( page.getMarginRight( ), 0 );

		contentWidth = pageWidth - leftMargin - rightMargin;

		this.orientation = page.getOrientation( );

	}

	public void endCell( ICellContent cell )
	{
		adjustInline( );
		context.removeWidth( );
		writer.endTblCell( context.needEmptyP( ) );
		context.endCell( );
	}

	public void endContainer( IContainerContent container )
	{
		boolean flag = false;

		IContainerContent con = container;

		while ( con != null )
		{

			if ( con.getParent( ) instanceof IForeignContent )
			{
				flag = true;
				break;
			}
			else
			{
				con = (IContainerContent) con.getParent( );
			}
		}

		if ( flag )
		{
			if ( !CSSConstants.CSS_INLINE_VALUE.equalsIgnoreCase( container
					.getComputedStyle( ).getDisplay( ) ) )
			{
				adjustInline( );
			}
			if ( !styles.isEmpty( ) )
			{
				styles.pop( );
			}

			if ( !CSSConstants.CSS_INLINE_VALUE.equalsIgnoreCase( container
					.getComputedStyle( ).getDisplay( ) ) )
			{
				if ( inForeign && hasPInside )
				{
					context.addContainer( false );
					hasPInside = false;
				}
				else
				{
					context.addContainer( true );
				}
				context.setLast( true );
			}
		}
	}

	public void startContainer( IContainerContent container )
	{
		boolean flag = false;

		IContainerContent con = container;

		while ( con != null )
		{

			if ( con.getParent( ) instanceof IForeignContent )
			{
				flag = true;
				break;
			}
			else
			{
				con = (IContainerContent) con.getParent( );
			}
		}

		if ( flag )
		{

			if ( !CSSConstants.CSS_INLINE_VALUE.equalsIgnoreCase( container
					.getComputedStyle( ).getDisplay( ) ) )
			{
				adjustInline( );
			}

			if ( context.isLastTable( ) )
			{
				writer.insertHiddenP( );
			}

			if ( !CSSConstants.CSS_INLINE_VALUE.equalsIgnoreCase( container
					.getComputedStyle( ).getDisplay( ) ) )
			{

				styles.push( container.getComputedStyle( ) );
				IStyle style = computeStyle( container.getComputedStyle( ) );

			}
		}
	}

	public void endContent( IContent content )
	{
		// TODO Auto-generated method stub

	}

	public void endGroup( IGroupContent group )
	{
		decreaseTOCLevel( group );
	}

	public void endList( IListContent list )
	{
		if ( !styles.isEmpty( ) )
		{
			styles.pop( );
		}

		context.addContainer( true );
		writer.endTable( );
		context.setLast( true );
		decreaseTOCLevel( list );
	}

	public void endListBand( IListBandContent listBand )
	{
		adjustInline( );
		context.endCell( );
		writer.endTblCell( true );
		writer.endTblRow( );
	}

	public void endListGroup( IListGroupContent group )
	{
		decreaseTOCLevel( group );
	}

	public void endPage( IPageContent page )
	{
	}

	public void endRow( IRowContent row )
	{
		if ( !isHidden( row ) )
		{
			if ( !styles.isEmpty( ) )
			{
				styles.pop( );
			}

			int col = context.getCurrentTblCols( ).length - 1;

			List spans = context.getSpans( col );

			if ( spans != null )
			{
				int spanSize = spans.size( );
				if ( spanSize > 0 )
				{
					rowFilledFlag = true;
				}
				for ( int i = 0; i < spanSize; i++ )
				{
					writer.writeSpanCell( (SpanInfo) spans.get( i ) );
				}
			}

			if ( !rowFilledFlag )
			{
				writer.writeEmptyCell( );
				rowFilledFlag = true;
			}

			writer.endTblRow( );
		}
	}

	public void endTable( ITableContent table )
	{
		context.addContainer( true );
		hasPInside = false;

		if ( !styles.isEmpty( ) )
		{
			styles.pop( );
		}

		writer.endTable( );
		context.setLast( true );
		context.removeTable( );
		decreaseTOCLevel( table );
	}

	public void endTableBand( ITableBandContent band )
	{
		// TODO Auto-generated method stub

	}

	public void endTableGroup( ITableGroupContent group )
	{
		decreaseTOCLevel( group );
	}

	public void start( IReportContent report )
	{
		this.reportContent = report;

	}

	public void startAutoText( IAutoTextContent autoText )
	{
		writeContent( autoText.getType( ), null, autoText );
	}

	public void startCell( ICellContent cell )
	{
		rowFilledFlag = true;
		context.startCell( );
		int col = cell.getColumn( );
		List spans = context.getSpans( col );

		if ( spans != null )
		{
			for ( int i = 0; i < spans.size( ); i++ )
			{
				writer.writeSpanCell( (SpanInfo) spans.get( i ) );
			}
		}

		// for(int i = 0; i < cacheCells.size( ); i++)
		// {
		// SpanCell scell = (SpanCell)cacheCells.get( i );
		//			
		// if(scell.cell.getRowSpan( ) > 1)
		// {
		// context.addSpan( scell.col, scell.cell.getColSpan( ), scell.cw,
		// scell.rowSpan, scell.style
		// }
		//			
		// }

		int cs = cell.getColSpan( );
		int rs = cell.getRowSpan( );
		int cw = context.getCellWidth( col, cs );

		IStyle style = computeStyle( cell.getComputedStyle( ) );

		if ( rs > 1 )
		{
			context.addSpan( col, cs, cw, rs, style );
			cacheCells.add( new SpanCell( cell, rs, col, cw, style ) );
		}

		SpanInfo info = null;

		if ( ( cs > 1 ) || ( rs > 1 ) )
		{
			info = new SpanInfo( col, cs, cw, true, style );
		}

		writer.startTblCell( cw, style, info );
		context.addWidth( WordUtil.getWidth( cw, style ) );
		writeTableToc();
	}

	private IStyle computeStyle( IStyle style )
	{
		if ( styles.size( ) == 0 )
		{
			return style;
		}

		for ( int i = 0; i < style.NUMBER_OF_STYLE; i++ )
		{
			if ( isHeritProperty( i ) )
			{
				if ( isNullValue( style.getProperty( i ) ) )
				{
					style.setProperty( i, null );

					for ( int p = styles.size( ) - 1; p >= 0; p-- )
					{
						IStyle pstyle = (IStyle) styles.get( p );

						if ( !isNullValue( pstyle.getProperty( i ) ) )
						{
							style.setProperty( i, pstyle.getProperty( i ) );
							break;
						}
					}
				}
			}
		}

		return style;
	}

	private boolean isNullValue( CSSValue value )
	{
		String[] nullvals = new String[]{"none", "transparent"};

		if ( value == null )
		{
			return true;
		}

		for ( int i = 0; i < nullvals.length; i++ )
		{
			if ( nullvals[i].equalsIgnoreCase( value.getCssText( ) ) )
			{
				return true;
			}
		}

		return false;
	}

	private boolean isHeritProperty( int p )
	{
		int[] ignores = new int[]{IStyle.STYLE_BORDER_BOTTOM_COLOR,
				IStyle.STYLE_BORDER_BOTTOM_STYLE,
				IStyle.STYLE_BORDER_BOTTOM_WIDTH,
				IStyle.STYLE_BORDER_TOP_COLOR, IStyle.STYLE_BORDER_TOP_STYLE,
				IStyle.STYLE_BORDER_TOP_WIDTH, IStyle.STYLE_BORDER_LEFT_COLOR,
				IStyle.STYLE_BORDER_LEFT_STYLE, IStyle.STYLE_BORDER_LEFT_WIDTH,
				IStyle.STYLE_BORDER_RIGHT_COLOR,
				IStyle.STYLE_BORDER_RIGHT_STYLE,
				IStyle.STYLE_BORDER_RIGHT_WIDTH};

		for ( int i = 0; i < ignores.length; i++ )
		{
			if ( p == ignores[i] )
			{
				return false;
			}
		}

		return true;
	}

	public void startContent( IContent content )
	{
		// TODO Auto-generated method stub
	}

	public void startData( IDataContent data )
	{
		writeContent( WordEmitterImpl.NORMAL, data.getText( ), data );
	}

	public void startForeign( IForeignContent foreign )
	{

		if ( IForeignContent.HTML_TYPE.equalsIgnoreCase( foreign.getRawType( ) ) )
		{

			inForeign = true;
			// store the inline state before the HTML foreign.
			boolean inlineBrother = context.isInline( );
			// the inline state needs be recalculated in the HTML foreign.
			context.endInline( );

			HTML2Content.html2Content( foreign );
			context.startCell( );

			if ( context.isLastTable( ) )
			{
				writer.insertHiddenP( );
			}

			writer.startTable( foreign.getComputedStyle( ), context
					.getCurrentWidth( ) );
			writer.startTblRow( -1 );
			writer.startTblCell( context.getCurrentWidth( ), foreign
					.getComputedStyle( ), null );
			writeToc( foreign );
			contentVisitor.visitChildren( foreign, null );

			adjustInline( );

			writer.endTblCell( context.needEmptyP( ) );

			context.endCell( );
			writer.endTblRow( );
			writer.endTable( );
			context.setLast( true );
			context.addContainer( true );
			hasPInside = false;
			// restore the inline state after the HTML foreign.
			if ( inlineBrother )
			{
				context.startInline( );
			}
			inForeign = false;
		}
		else
		{
			Object rawValue = foreign.getRawValue( );
			String text = rawValue == null ? "" : rawValue.toString( );
			writeContent( WordEmitterImpl.NORMAL, text, foreign );
		}
	}

	public void startGroup( IGroupContent group )
	{
		setGroupToc( group );
	}

	public void startImage( IImageContent image )
	{
		String uri = image.getURI( );
		String mimeType = image.getMIMEType( );
		String extension = image.getExtension( );
		boolean inline = false;
		boolean first_inline = false;
		IStyle style = image.getComputedStyle( );
		if ( "inline".equalsIgnoreCase( style.getDisplay( ) ) )
		{
			if ( !context.isInline( ) )
			{
				context.startInline( );
				first_inline = true;
			}
			inline = true;
		}
		else
		{
			adjustInline( );
		}

		if ( FlashFile.isFlash( mimeType, uri, extension ) )
		{
			String altText = image.getAltText( );
			if ( null == altText )
			{
				if ( null != eservice )
				{
					IReportContext reportContext = eservice.getReportContext();
					if ( null != reportContext ) 
					{
						ULocale locale = ULocale.forLocale( reportContext
								.getLocale( ) );
						EngineResourceHandle handle = new EngineResourceHandle( locale );
						altText = handle
								.getMessage( MessageConstants.FLASH_OBJECT_NOT_SUPPORTED_PROMPT );
					}
				}
			}
			writer.drawImage( null, 0.0, 0.0, null, style, inline,
					first_inline, altText );
			return;
		}

		byte[] data = parseImage( image );
		if ( data == null )
		{
			writer.drawImage( null, 0.0, 0.0, null, style, inline,
					first_inline, image.getAltText( ) );
			return;
		}

		int defaultW = 0;
		int defaultH = 0;
		try
		{
			Image imageData = javax.imageio.ImageIO
					.read( new ByteArrayInputStream( data ) );

			if ( imageData != null )
			{
				defaultW = imageData.getWidth( null );// pix

				defaultH = imageData.getHeight( null );
			}
		}
		catch ( Exception e )
		{
			logger.log( Level.WARNING, e.getMessage( ), e );
		}

		double height = WordUtil
				.convertImageSize( image.getHeight( ), defaultH );

		double width = WordUtil.convertImageSize( image.getWidth( ), defaultW );

		// //1 inch = 25.4mm 72pt = 1 inch
		// pt is absolute measure different from relative measure px

		String pic2Text = new sun.misc.BASE64Encoder( ).encode( data );

		writeBookmark( image );
		writeToc( image );
		HyperlinkInfo hyper = getHyperlink( image );

		writer.drawImage( pic2Text, height, width, hyper, style, inline,
				first_inline, image.getAltText( ) );
	}

	public void startLabel( ILabelContent label )
	{
		String txt = label.getText( ) == null ? label.getLabelText( ) : label
				.getText( );
		String top = label.getComputedStyle( ).getMarginTop( );
		String bottom = label.getComputedStyle( ).getMarginBottom( );
		txt = txt == null ? "" : txt;
		writeContent( WordEmitterImpl.NORMAL, txt, label );
	}

	private IReportContext getReportContext( )
	{
		if ( eservice != null )
		{
			return eservice.getReportContext( );
		}
		return null;
	}

	public void startList( IListContent list )
	{
		adjustInline( );

		styles.push( list.getComputedStyle( ) );

		writeBookmark( list );
		Object listToc = list.getTOC( );
		if ( listToc != null )
		{
			tableTocs.add( new TocInfo( listToc.toString( ), tocLevel ) );
		}
		increaseTOCLevel( list );

		if ( context.isLastTable( ) )
		{
			writer.insertHiddenP( );
		}

		writer
				.startTable( list.getComputedStyle( ), context
						.getCurrentWidth( ) );
	}

	public void startListBand( IListBandContent listBand )
	{
		context.startCell( );
		writer.startTblRow( -1 );

		IStyle style = computeStyle( listBand.getComputedStyle( ) );
		writer.startTblCell( context.getCurrentWidth( ), style, null );
		writeTableToc( );
	}

	public void startListGroup( IListGroupContent group )
	{
		setGroupToc( group );
	}

	public void startRow( IRowContent row )
	{
		if ( !isHidden( row ) )
		{
			rowFilledFlag = false;
			boolean isHeader = false;
			styles.push( row.getComputedStyle( ) );
			if ( row.getBand( ) != null
					&& row.getBand( ).getBandType( ) == IBandContent.BAND_HEADER )
			{
				isHeader = true;
			}

			double height = WordUtil.convertTo( row.getHeight( ) );

			writer.startTblRow( height, isHeader, row.getTable( )
					.isHeaderRepeat( ) );
			context.newRow( );
		}
	}

	protected boolean isHidden( IContent content )
	{
		if ( content != null )
		{
			IStyle style = content.getStyle( );
			if ( !IStyle.NONE_VALUE.equals( style
					.getProperty( IStyle.STYLE_DISPLAY ) ) )
			{
				return isHiddenByVisibility( content );
			}
			return true;
		}
		return false;
	}

	/**
	 * if the content is hidden
	 * 
	 * @return
	 */
	private boolean isHiddenByVisibility( IContent content )
	{
		assert content != null;
		IStyle style = content.getStyle( );
		String formats = style.getVisibleFormat( );
		if ( contains( formats, OUTPUT_FORMAT ) )
		{
			return true;
		}
		return false;
	}

	private boolean contains( String formats, String format )
	{
		if ( formats != null
				&& ( formats.indexOf( EngineIRConstants.FORMAT_TYPE_VIEWER ) >= 0
						|| formats.indexOf( BIRTConstants.BIRT_ALL_VALUE ) >= 0 || formats
						.indexOf( format ) >= 0 ) )
		{
			return true;
		}
		return false;
	}

	public void startTable( ITableContent table )
	{
		adjustInline( );

		styles.push( table.getComputedStyle( ) );

		IStyle style = table.getComputedStyle( );

		writeBookmark( table );
		Object tableToc = table.getTOC( );
		if ( tableToc != null )
		{
			tableTocs.add( new TocInfo( tableToc.toString( ), tocLevel ) );
		}
		increaseTOCLevel( table );

		String caption = table.getCaption( );
		if ( caption != null )
		{
			writer.writeCaption( caption );
		}

		if ( context.isLastTable( ) )
		{
			writer.insertHiddenP( );
		}

		int tblWid = WordUtil.convertTo( table.getWidth( ), context
				.getCurrentWidth( ) );

		tblWid = Math.min( tblWid, context.getCurrentWidth( ) );

		writer.startTable( table.getComputedStyle( ), tblWid );

		int colCount = table.getColumnCount( );
		int[] cols = new int[colCount];
		int count = 0;
		int total = 0;

		for ( int i = 0; i < colCount; i++ )
		{
			IColumn col = table.getColumn( i );
			if ( col.getWidth( ) == null )
			{
				cols[i] = -1;
				count++;
			}
			else
			{
				cols[i] = WordUtil.convertTo( col.getWidth( ), tblWid );
				total += cols[i];
			}
		}

		for ( int i = 0; i < cols.length; i++ )
		{
			if ( cols[i] == -1 )
			{
				cols[i] = ( tblWid - total ) / count;
			}
		}

		writer.writeCol( cols );

		context.addTable( cols, style );
	}

	public void startTableBand( ITableBandContent band )
	{

	}

	public void startTableGroup( ITableGroupContent group )
	{
		setGroupToc( group );
	}

	private void setGroupToc( IGroupContent group )
	{
		if ( group != null )
		{
			InstanceID groupId = group.getInstanceID( );
			if ( !groupIdList.contains( groupId ) )
			{
				groupIdList.add( groupId );
				Object groupToc = group.getTOC( );
				if ( groupToc != null )
				{
					tableTocs
							.add( new TocInfo( groupToc.toString( ), tocLevel ) );
				}
			}
			increaseTOCLevel( group );
		}
	}

	private void writeTableToc( )
	{
		if ( !tableTocs.isEmpty( ) )
		{
			for ( TocInfo toc : tableTocs )
			{
				if ( !"".equals( toc.tocValue ) )
				{
					writer.writeTC( toc.tocValue, toc.tocLevel );
				}
			}
			tableTocs.clear( );
		}
	}
	
	public void startText( ITextContent text )
	{
		writeContent( WordEmitterImpl.NORMAL, text.getText( ), text );
	}

	// TOC must not contain space,word may not process TOC with
	// space
	private void writeToc( IContent content )
	{
		if ( content != null )
		{
			Object tocObj = content.getTOC( );
			if ( tocObj != null )
			{
				String toc = tocObj.toString( );
				toc = toc.trim( );

				if ( !"".equals( toc ) )
				{
					writer.writeTC( toc, tocLevel );
				}
			}
		}
	}

	private void writeBookmark( IContent content )
	{
		Object design = content.getGenerateBy( );

		if ( marks.contains( design ) )
		{
			return;
		}

		String bookmark = content.getBookmark( );
		// birt use __TOC_X_X as bookmark for toc and thus it is not a
		// really
		// bookmark
		if ( bookmark == null || bookmark.toLowerCase( ).indexOf( "_toc" ) > 0 )
		{
			return;
		}

		bookmark = bookmark.replaceAll( " ", "_" );
		writer.writeBookmark( bookmark );
		marks.add( design );
	}

	protected HyperlinkInfo getHyperlink( IContent content )
	{
		HyperlinkInfo hyperlink = null;
		IHyperlinkAction linkAction = content.getHyperlinkAction( );
		if ( linkAction != null )
		{
			String tooltip = linkAction.getTooltip( );
			String bookmark = linkAction.getBookmark( );
			IReportRunnable reportRunnable = eservice.getReportRunnable( );
			IReportContext reportContext = eservice.getReportContext( );
			IHTMLActionHandler actionHandler = (IHTMLActionHandler) eservice
					.getOption( RenderOption.ACTION_HANDLER );
			switch ( linkAction.getType( ) )
			{
				case IHyperlinkAction.ACTION_BOOKMARK :
					bookmark = bookmark.replaceAll( " ", "_" );
					hyperlink = new HyperlinkInfo( HyperlinkInfo.BOOKMARK,
							bookmark, tooltip );
					break;
				case IHyperlinkAction.ACTION_HYPERLINK :
				case IHyperlinkAction.ACTION_DRILLTHROUGH :
					String url = org.eclipse.birt.report.engine.layout.emitter.EmitterUtil
							.getHyperlinkUrl( linkAction, reportRunnable,
									actionHandler, reportContext );
					hyperlink = new HyperlinkInfo( HyperlinkInfo.HYPERLINK,
							url, tooltip );
					break;
			}
		}
		return hyperlink;
	}

	private byte[] getImage( InputStream is ) throws Exception
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream( );

		int cnt;
		byte[] buf = new byte[4096];

		while ( ( cnt = is.read( buf ) ) >= 0 )
		{
			baos.write( buf, 0, cnt );
		}
		is.close( );
		baos.close( );
		return baos.toByteArray( );

	}

	private byte[] parseImage( IImageContent image )
	{
		byte[] data = null;
		InputStream in = null;
		String uri = image.getURI( );

		int source = image.getImageSource( );

		try
		{
			if ( IImageContent.IMAGE_URL == source
					|| IImageContent.IMAGE_FILE == source )
			{
				if ( uri != null )
				{
					URL url = new URL( uri );
					in = url.openStream( );

					if ( SvgFile.isSvg( uri ) )
					{
						data = SvgFile.transSvgToArray( in );
					}
					else
					{
						data = getImage( in );
					}
				}
			}
			else if ( source == IImageContent.IMAGE_EXPRESSION
					|| source == IImageContent.IMAGE_NAME )
			{
				if ( SvgFile.isSvg( image.getMIMEType( ), uri, image
						.getExtension( ) ) )
				{
					in = new ByteArrayInputStream( image.getData( ) );
					data = SvgFile.transSvgToArray( in );
				}
				else
				{
					data = image.getData( );
				}
			}
		}
		catch ( Exception e )
		{
			logger.log( Level.WARNING, e.getMessage( ) );
		}
		finally
		{
			if ( in != null )
			{
				try
				{
					in.close( );
					in = null;
				}
				catch ( Exception e )
				{
				}
			}
		}

		return data;
	}

	private void writeContent( int type, String txt, IContent content )
	{
		if ( inForeign )
		{
			hasPInside = true;
		}
		context.addContainer( false );

		boolean inline = false;
		boolean first_inline = false;
		IStyle c_style = content.getComputedStyle( );

		if ( "inline".equalsIgnoreCase( content.getComputedStyle( )
				.getDisplay( ) ) )
		{
			if ( !context.isInline( ) )
			{
				context.startInline( );
				first_inline = true;
				if ( !styles.isEmpty( ) )
				{
					c_style = (ComputedStyle) styles.peek( );
				}
			}
			inline = true;
		}
		else
		{
			adjustInline( );
		}

		writeBookmark( content );
		writeToc( content ); // element with Toc contains bookmark
		HyperlinkInfo hyper = getHyperlink( content );

		if ( content instanceof TextContent )
		{
			TextFlag flag = TextFlag.start;
			String fontFamily = null;
			if ( "".equals( txt ) )
			{
				writer.writeContent( type, txt, c_style, fontFamily, hyper,
						inline, first_inline, flag );
			}
			else
			{
				FontSplitter fontSplitter = getFontSplitter( content );
				while ( fontSplitter.hasMore( ) )
				{
					Chunk ch = fontSplitter.getNext( );
					FontInfo info = ch.getFontInfo( );
					BaseFont basefont = null;
					if ( info != null )
					{
						basefont = info.getBaseFont( );
						String[][] fontName = basefont.getFamilyFontName( );
						int x = fontName.length - 1;
						int y = fontName[0].length - 1;
						fontFamily = fontName[x][y];
					}
					else
					{
						fontFamily = c_style.getFontFamily( );
					}
					writer.writeContent( type, ch.getText( ), c_style,
							fontFamily, hyper, inline, first_inline, flag );
					if ( !fontSplitter.hasMore( ) )
					{
						flag = TextFlag.end;
					}
					else
					{
						flag = TextFlag.middle;
					}
				}
			}
			if ( inline == false )
			{
				writer.writeContent( type, null, c_style, fontFamily, hyper,
						inline, first_inline, TextFlag.end );
			}
		}
		else
		{
			writer.writeContent( type, txt, c_style, c_style.getFontFamily( ),
					hyper, inline, first_inline, TextFlag.whole );
		}

		context.setLast( false );
	}

	private FontSplitter getFontSplitter( IContent content )
	{
		FontMappingManager fontManager = FontMappingManagerFactory
				.getInstance( ).getFontMappingManager( "doc",
						Locale.getDefault( ) );
		String text = ( (TextContent) content ).getText( );
		FontSplitter fontSplitter = new FontSplitter( fontManager, new Chunk(
				text ), (TextContent) content, true );
		return fontSplitter;
	}

	private void adjustInline( )
	{
		if ( context.isInline( ) )
		{
			writer.endParagraph( );
			context.endInline( );
		}
	}

	class SpanCell
	{

		SpanCell( ICellContent cell, int rowSpan, int col, int cw, IStyle style )
		{
			this.cell = cell;
			this.rowSpan = rowSpan;
			this.col = col;
			this.cw = cw;
			this.style = style;
		}

		ICellContent cell;

		int rowSpan, col, cw;

		IStyle style;
	}

	class TocInfo
	{

		String tocValue;
		int tocLevel;

		TocInfo( String tocValue, int tocLevel )
		{
			this.tocValue = tocValue;
			this.tocLevel = tocLevel;
		}
	}
}
